uniform mat4	uniViewProjectionMatrix;
uniform vec4	uniColor;	//	premultiplied alpha
uniform vec3	uniDiffuseEvaluator,
				uniSpecularEvaluator;
uniform float	uniAmbientFactor,
				uniDiffuseFactor,
				uniSpecularFactor;

in mat3			atrModelMatrix;
in vec3			atrPosition;
in vec3			atrNormal;

out vec4		varColor;	//	premultiplied alpha


void main()
{
	vec3	tmpNormal;
	float	tmpDiffuseFactor,
			tmpSpecularFactor;

	gl_Position = uniViewProjectionMatrix * vec4(atrModelMatrix * atrPosition, 1.0);

	//	Strictly speaking the surface normal is a 1-form, not a vector,
	//	but for rigid motions in Euclidean space, 1-forms and vectors
	//	transform the same way.
	tmpNormal			= atrModelMatrix * atrNormal;
	tmpDiffuseFactor	= uniAmbientFactor + uniDiffuseFactor * max(0.0, dot(uniDiffuseEvaluator, tmpNormal));
	tmpSpecularFactor	= uniSpecularFactor * pow(max(0.0, dot(uniSpecularEvaluator, tmpNormal)), 64.0);

	varColor = vec4(tmpDiffuseFactor * uniColor.r  +  tmpSpecularFactor,
					tmpDiffuseFactor * uniColor.g  +  tmpSpecularFactor,
					tmpDiffuseFactor * uniColor.b  +  tmpSpecularFactor,
									   uniColor.a  +  tmpSpecularFactor);
}
